/* 
 * Copyright (C) 2010 by Björn Biesenbach <bjoern@bjoern-b.de>
 *
 * This file is part of dockextd
 *
 * dockextd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * dockextd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with dockextd.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <glib.h>
#include <stdint.h>
#include <time.h>
#include <stdio.h>
#include <string.h>

#define NUM_MENUITEMS (sizeof(menu)/sizeof(struct menuitem))
#define CONFIG_FILE "/etc/dockextd.conf"

static uint8_t backlight = 64;
static int menu_position = 0;
static int submenu_position = 0;
static GMainLoop *main_loop;

static char *mountpoints[] = {"/", "/mnt/usb1"};

void increment_menu(void);
void decrement_menu(void);
void increment_backlight(void);
void decrement_backlight(void);
void setTextIpAddress(void);
void setTextDiskFree(void);
char *getipaddress(char *device);
void setTextBacklight(void);
char *getDiskFree(char *mountpoint);
void increment_df(void);
void decrement_df(void);
void ds_shutdown(void);

struct menuitem{
    const char *name;
    int auto_refresh;
    void (*t1_action)();
    void (*t2_action)();
    void (*t3_action)();
    void (*t4_action)();
    void (*draw_func)();
};

static const struct menuitem menu[] = {
    {"Dockext",1,NULL,decrement_menu, increment_menu, NULL, setTextIpAddress},
    {"Beleuchtung",0, decrement_backlight,decrement_menu, increment_menu, increment_backlight, setTextBacklight},
    {"Freier Speicher",1,decrement_df,decrement_menu, increment_menu, increment_df, setTextDiskFree},
    {"Unmount",0,NULL,decrement_menu, increment_menu, NULL, NULL},
    {"Herunterfahren",0,ds_shutdown,decrement_menu, increment_menu, ds_shutdown, NULL}
};

void ds_shutdown(){
    system("init 0");
    serialSetText("Fahre herunter",2,1);
}

void increment_df(){
    if(++submenu_position > (sizeof(mountpoints)/sizeof(char*))-1)
        submenu_position = 0;
    setTextDiskFree();
}

void decrement_df(){
    if(--submenu_position < 0)
        submenu_position = (sizeof(mountpoints)/sizeof(char*))-1;
    setTextDiskFree();
}

void setTextIpAddress(){
    char textline[17];
    snprintf(textline,16,"%s",getipaddress("eth0"));
    serialSetText(textline,2,1);
}

void setTextDiskFree(){
    char textline[17];
    sprintf(textline,"%s",getDiskFree(mountpoints[submenu_position]));
    serialSetText(textline,2,1);
}

void setTextBacklight(void){
    char buf[17];
    if(backlight == 255)
        strcpy(buf,"4/4");
    else
        sprintf(buf,"%d/4",backlight/64);
    serialSetText(buf,2,1);
}

void increment_menu(void){
    submenu_position = 0;
    if(++menu_position > NUM_MENUITEMS-1)
        menu_position = 0;
    serialSetText(menu[menu_position].name,1,1);
    if(menu[menu_position].draw_func)
        menu[menu_position].draw_func();
    else
        serialSetText("",2,1);
}

void decrement_menu(void){
    submenu_position = 0;
    if(--menu_position < 0)
        menu_position = NUM_MENUITEMS -1;
    serialSetText(menu[menu_position].name,1,1);
    if(menu[menu_position].draw_func)
        menu[menu_position].draw_func();
    else
        serialSetText("",2,1);
}

void increment_backlight(void){
    backlight +=64;
    if(backlight == 0)
        backlight = 255;
    if(backlight == 63)
        backlight = 0;
    serialSetBacklight(backlight);
    setTextBacklight();
}

void decrement_backlight(void){
    backlight -=64;
    if(backlight == 255)
        backlight = 0;
    if(backlight == 192)
        backlight = 255;
    serialSetBacklight(backlight);
    setTextBacklight();
}

char *getipaddress(char *device){
    FILE *pifconfig;
    char line[255];
    char cmd[255];
    char *tmp;
    static char address[20];
    
    g_assert(device);
    
    address[0] = '\0';
    sprintf(cmd,"/sbin/ifconfig %s",device);
    pifconfig = popen(cmd,"r");
    
    while(fgets(line, sizeof(line), pifconfig)){
        if((tmp=strstr(line,"inet addr:"))){
            tmp = strchr(tmp,':')+1;
            strncpy(address,tmp,sizeof(address));
            address[sizeof(address)-1] = '\0';
            tmp = strchr(address,' ');
            *tmp = '\0';
        }
    }
    pclose(pifconfig);
    return address;
}

char *getDiskFree(char *mountpoint){
    FILE *pdf;
    static char line[255];
    char cmd[255];
    char *tmp;
    char *split;
    char space_free[10], space_used[10], space_size[10];

    g_assert(mountpoint);
    
    line[0] = '\0';
    sprintf(cmd,"/bin/df -h %s",mountpoint);
    pdf = popen(cmd,"r");
    
    while(fgets(line, sizeof(line), pdf)){
        if((tmp=strstr(line,"/"))){
            split = strtok(tmp," ");
            split = strtok(NULL," ");
            strncpy(space_size,split,sizeof(space_size));
            split = strtok(NULL," ");
            strncpy(space_used,split,sizeof(space_used));
            split = strtok(NULL," ");
            strncpy(space_free,split,sizeof(space_free));
        }
    }
    snprintf(line,16,"%s/%s",space_free,space_size);
    line[16] = '\0';
    pclose(pdf);
    return line;
}

void taster1(){
    if(menu[menu_position].t1_action)
        menu[menu_position].t1_action();
}

void taster2(){
    if(menu[menu_position].t2_action)
        menu[menu_position].t2_action();
}

void taster3(){
    if(menu[menu_position].t3_action)
        menu[menu_position].t3_action();
}

void taster4(){
    if(menu[menu_position].t4_action)
        menu[menu_position].t4_action();
}

gboolean tenSecondTimer(gpointer data){

    struct tm *ptm;
    time_t now;
    char textline[33];

    now = time(NULL);
    ptm = localtime(&now);

    //sprintf(textline,"%s    %02d:%02d:%02d",menu[0],ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
    if(menu[menu_position].auto_refresh){
        if(menu[menu_position].draw_func)
            menu[menu_position].draw_func();
        else
            serialSetText("",2,1);
    }
    return TRUE;
}

void signalHandler(int sig){

    if(sig == SIGTERM || sig == SIGKILL){
        menu_position = -1;
        serialSetText("Schalt mich aus",2,1);
    }
    g_main_loop_quit(main_loop);
}

void displayReady(){
    serialSetBacklight(backlight);
    serialSetText(menu[menu_position].name,1,1);
    if(menu[menu_position].draw_func)
        menu[menu_position].draw_func();
    else
        serialSetText("",2,0);
}

int main(int argc, char *argv[]){
    
    GKeyFile *key_file;
    GError *error = NULL;
    FILE *key_output_file;
    gchar *content;
    gsize length;

    serialInit();
    key_file = g_key_file_new();
    g_key_file_load_from_file(key_file, CONFIG_FILE,
        G_KEY_FILE_KEEP_COMMENTS, NULL);

    backlight = g_key_file_get_integer(key_file, "general", "backlight", NULL);

    signal(SIGTERM, signalHandler);
    signal(SIGKILL, signalHandler);

    serialSetCallback(1,taster1);
    serialSetCallback(2,taster2);
    serialSetCallback(3,taster3);
    serialSetCallback(4,taster4);
    serialSetCallbackReady(displayReady);

    g_timeout_add_seconds(10, tenSecondTimer, NULL);
    main_loop = g_main_loop_new(NULL,0);
    g_main_loop_run(main_loop);
    
    g_key_file_set_integer(key_file, "general", "backlight", backlight);
    key_output_file = fopen(CONFIG_FILE,"w");
    if(key_output_file){
        content = g_key_file_to_data(key_file, &length, NULL);
        fwrite(content, length, 1, key_output_file);
        fclose(key_output_file);
        g_free(content);
    }
    return 0;
}
